/*
 *   This file is part of Dianara
 *   Copyright 2012  JanKusanagi <janjabber@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .
 */

#include "account.h"

AccountDialog::AccountDialog(QWidget *parent) :  QWidget(parent)
{
    this->setWindowTitle("Dianara - " + tr("Account configuration"));
    this->setWindowIcon(QIcon::fromTheme("user-properties"));
    this->setMinimumSize(540, 150);

    userIDIconLabel = new QLabel();
    userIDIconLabel->setPixmap(QIcon::fromTheme("preferences-desktop-user").pixmap(32,32));
    userIDLabel = new QLabel(tr("Your Diaspora ID:"));
    userIDLineEdit = new QLineEdit();
    userIDLineEdit->setPlaceholderText(tr("Ex: username@podname.org"));

    accessTokenIconLabel = new QLabel();
    accessTokenIconLabel->setPixmap(QIcon::fromTheme("dialog-password").pixmap(32,32));
    accessTokenLabel = new QLabel(tr("Your access token:"));
    accessTokenLineEdit = new QLineEdit();
    accessTokenLineEdit->setPlaceholderText(tr("Enter the access code provided by Diaspora here"));


    // To notify invalid ID or empty access token
    // Cleared when typing in any of the two fields
    errorsLabel = new QLabel("");
    connect(userIDLineEdit, SIGNAL(textChanged(QString)),
            errorsLabel, SLOT(clear()));
    connect(accessTokenLineEdit, SIGNAL(textChanged(QString)),
            errorsLabel, SLOT(clear()));

    saveButton = new QPushButton(QIcon::fromTheme("document-save"),
                                 tr("&Save details"));
    connect(saveButton, SIGNAL(clicked()),
            this, SLOT(saveDetails()));
    cancelButton = new QPushButton(QIcon::fromTheme("dialog-cancel"),
                                   tr("&Cancel"));
    connect(cancelButton, SIGNAL(clicked()),
            this, SLOT(close()));

    mainLayout = new QGridLayout();
    mainLayout->setAlignment(Qt::AlignLeft);
    mainLayout->addWidget(userIDIconLabel,   0, 0);
    mainLayout->addWidget(userIDLabel,       0, 1);
    mainLayout->addWidget(userIDLineEdit,    0, 2);

    mainLayout->addWidget(accessTokenIconLabel, 1, 0);
    mainLayout->addWidget(accessTokenLabel,     1, 1);
    mainLayout->addWidget(accessTokenLineEdit,  1, 2);

    mainLayout->addWidget(errorsLabel,       2, 0, 1, 3, Qt::AlignCenter);

    mainLayout->addWidget(saveButton,        3, 1, 1, 1, Qt::AlignRight);
    mainLayout->addWidget(cancelButton,      3, 2, 1, 1, Qt::AlignRight);
    this->setLayout(mainLayout);

    QSettings settings;
    userIDLineEdit->setText(settings.value("userID", "").toString());
    accessTokenLineEdit->setText(settings.value("userAuth", "").toString());

    qDebug() << "Account dialog created";
}


AccountDialog::~AccountDialog()
{
    qDebug() << "Account dialog destroyed";
}



void AccountDialog::saveDetails()
{
    // Don't fail if there're spaces before or after the ID, they're only human ;)
    userIDLineEdit->setText(userIDLineEdit->text().trimmed());

    // Match username@server.tld AND check for only 1 @ sign
    if (QRegExp("\\S+@\\S+\\.\\S+").exactMatch(this->userIDLineEdit->text())
     && userIDLineEdit->text().count("@") == 1)
    {
        if (this->accessTokenLineEdit->text().length() > 0)
        {
            // Both correct, go!

            QSettings settings;
            settings.setValue("userID", this->userIDLineEdit->text());
            settings.setValue("userAuth", accessTokenLineEdit->text());


            emit userIDChanged(userIDLineEdit->text(),
                               accessTokenLineEdit->text());

            this->errorsLabel->clear(); // Clear previous error messages, if any
            this->close();
        }
        else // access token is empty
        {
            this->errorsLabel->setText("[ " + tr("Access token is empty") + " ]");
            qDebug() << "OAuth access token is empty!";
        }
    }
    else // userID does not match user@hostname.domain
    {
        this->errorsLabel->setText("[ " + tr("Your Diaspora ID is invalid") + " ]");
        qDebug() << "userID is invalid!";
    }

}
